import _extends from "@babel/runtime/helpers/extends";
import _Switch from "antd/es/switch";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/objectWithoutPropertiesLoose";
var _excluded = ["className", "style", "title", "children"];
import React from 'react';
import { useField, observer } from '@formily/react';
import { FormItem, useFormLayout } from '@formily/antd-v5';
import cls from 'classnames';
import useStyle from './style';
import { usePrefixCls } from '../../../../__builtins__/hooks';
/**
 * Schema 驱动的开关对象字段组件
 *
 * 在 schema 中使用：
 * ```json
 * inputSearchProps: {
 *   type: 'object',
 *   'x-decorator': 'FormItem',
 *   'x-component': 'SwitchObjectField',
 *   'x-component-props': { title: '简易搜索' },
 *   properties: { ... 子字段配置 ... }
 * }
 * ```
 *
 * 当开关关闭时，字段值为 undefined；
 * 当开关开启时，字段值为包含子属性的对象。
 */
export var FormSwitchObject = observer(function (_ref) {
  var className = _ref.className,
    style = _ref.style,
    title = _ref.title,
    children = _ref.children,
    restProps = _objectWithoutPropertiesLoose(_ref, _excluded);
  var field = useField();
  console.log('-------', field, '-------', restProps);
  var value = field.value;
  var enabled = !!value;
  var prefixCls = usePrefixCls('form-switch-object', restProps);
  var _useStyle = useStyle(prefixCls),
    wrapSSR = _useStyle[0],
    hashId = _useStyle[1];
  // 切换开关：开启则初始化空对象，关闭则置为 undefined
  var handleToggle = function handleToggle(checked) {
    if (checked) {
      field.setValue({});
    } else {
      field.setValue(false);
    }
  };
  var deepLayout = useFormLayout();
  console.log('---deepLayout', deepLayout);
  return wrapSSR(/*#__PURE__*/React.createElement("div", {
    className: cls(prefixCls + "-layout", hashId, className),
    style: style
  }, /*#__PURE__*/React.createElement(FormItem, _extends({
    label: (field === null || field === void 0 ? void 0 : field.title) || '启用'
  }, deepLayout), /*#__PURE__*/React.createElement(_Switch, {
    checked: enabled,
    onChange: handleToggle
  })), enabled && children));
});