import _extends from "@babel/runtime/helpers/extends";
import _Tooltip from "antd/es/tooltip";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/objectWithoutPropertiesLoose";
var _excluded = ["className", "language", "defaultLanguage", "width", "helpLink", "helpCode", "helpCodeViewWidth", "height", "onMount", "onChange", "theme"];
import React, { useState, useRef, useEffect } from 'react';
import { Editor, loader } from '@monaco-editor/react';
import { parseExpression, parse } from '@babel/parser';
import { format } from './format';
import cls from 'classnames';
import { initMonaco } from './config';
import { uid } from '@formily/shared';
import { createIcon } from '@gant-lowcode/lowcode-utils';
import { usePrefixCls } from '@formily/antd-v5/lib/__builtins__';
import { useStyle } from './style';
export var MonacoInput = function MonacoInput(_ref) {
  var className = _ref.className,
    language = _ref.language,
    defaultLanguage = _ref.defaultLanguage,
    width = _ref.width,
    helpLink = _ref.helpLink,
    helpCode = _ref.helpCode,
    helpCodeViewWidth = _ref.helpCodeViewWidth,
    height = _ref.height,
    onMount = _ref.onMount,
    onChange = _ref.onChange,
    theme = _ref.theme,
    props = _objectWithoutPropertiesLoose(_ref, _excluded);
  var _useState = useState(false),
    loaded = _useState[0],
    setLoaded = _useState[1];
  var valueRef = useRef('');
  var validateRef = useRef(null);
  var submitRef = useRef(null);
  var declarationRef = useRef([]);
  var extraLibRef = useRef(null);
  var monacoRef = useRef();
  var editorRef = useRef();
  var computedLanguage = useRef(language || defaultLanguage);
  var realLanguage = useRef('');
  var unmountedRef = useRef(false);
  var changedRef = useRef(false);
  var uidRef = useRef(uid());
  var prefixCls = usePrefixCls('monaco-input', props);
  var _useStyle = useStyle(prefixCls),
    wrapSSR = _useStyle[0],
    hashId = _useStyle[1];
  var input = props.value || props.defaultValue;
  useEffect(function () {
    unmountedRef.current = false;
    initMonaco();
    return function () {
      if (extraLibRef.current) {
        extraLibRef.current.dispose();
      }
      unmountedRef.current = true;
    };
  }, []);
  useEffect(function () {
    if (monacoRef.current && props.extraLib) {
      updateExtraLib();
    }
  }, [props.extraLib]);
  var updateExtraLib = function updateExtraLib() {
    if (extraLibRef.current) {
      extraLibRef.current.dispose();
    }
    extraLibRef.current = monacoRef.current.languages.typescript.typescriptDefaults.addExtraLib(props.extraLib, uidRef.current + ".d.ts");
  };
  var isFileLanguage = function isFileLanguage() {
    var lang = computedLanguage.current;
    return lang === 'javascript' || lang === 'typescript';
  };
  var isExpLanguage = function isExpLanguage() {
    var lang = computedLanguage.current;
    return lang === 'javascript.expression' || lang === 'typescript.expression';
  };
  var renderHelper = function renderHelper() {
    var getHref = function getHref() {
      if (typeof helpLink === 'string') return helpLink;
      if (isFileLanguage()) {
        return 'https://developer.mozilla.org/zh-CN/docs/Web/JavaScript';
      }
      if (isExpLanguage()) {
        return 'https://developer.mozilla.org/zh-CN/docs/Web/JavaScript/Reference/Operators';
      }
    };
    if (helpLink === false) return null;
    var href = getHref();
    return href && /*#__PURE__*/React.createElement(_Tooltip, {
      title: "\u5E2E\u52A9\u6587\u6863"
    }, /*#__PURE__*/React.createElement("div", {
      className: prefixCls + "-helper"
    }, /*#__PURE__*/React.createElement("a", {
      target: "_blank",
      href: href,
      rel: "noreferrer"
    }, createIcon('wenhao'))));
  };
  var onMountHandler = function onMountHandler(editor, monaco) {
    editorRef.current = editor;
    monacoRef.current = monaco;
    onMount === null || onMount === void 0 ? void 0 : onMount(editor, monaco);
    var model = editor.getModel();
    var currentValue = editor.getValue();
    model['getDesignerLanguage'] = function () {
      return computedLanguage.current;
    };
    if (currentValue) {
      format(computedLanguage.current, currentValue).then(function (content) {
        editor.setValue(content);
        setLoaded(true);
      })["catch"](function () {
        setLoaded(true);
      });
    } else {
      setLoaded(true);
    }
    if (props.extraLib) {
      updateExtraLib();
    }
    editor.onDidChangeModelContent(function () {
      onChangeHandler(editor.getValue());
    });
  };
  var submit = function submit() {
    clearTimeout(submitRef.current);
    submitRef.current = setTimeout(function () {
      onChange === null || onChange === void 0 ? void 0 : onChange(valueRef.current);
    }, 1000);
  };
  var validate = function validate() {
    if (realLanguage.current === 'typescript') {
      clearTimeout(validateRef.current);
      validateRef.current = setTimeout(function () {
        try {
          if (valueRef.current) {
            if (isFileLanguage()) {
              parse(valueRef.current, {
                sourceType: 'module',
                plugins: ['typescript', 'jsx']
              });
            } else if (isExpLanguage()) {
              parseExpression(valueRef.current, {
                plugins: ['typescript', 'jsx']
              });
            }
          }
          monacoRef.current.editor.setModelMarkers(editorRef.current.getModel(), computedLanguage.current, []);
          declarationRef.current = editorRef.current.deltaDecorations(declarationRef.current, [{
            range: new monacoRef.current.Range(1, 1, 1, 1),
            options: {}
          }]);
          submit();
        } catch (e) {
          declarationRef.current = editorRef.current.deltaDecorations(declarationRef.current, [{
            range: new monacoRef.current.Range(e.loc.line, e.loc.column, e.loc.line, e.loc.column),
            options: {
              isWholeLine: true,
              glyphMarginClassName: 'monaco-error-highline'
            }
          }]);
          monacoRef.current.editor.setModelMarkers(editorRef.current.getModel(), computedLanguage.current, [{
            code: '1003',
            severity: 8,
            startLineNumber: e.loc.line,
            startColumn: e.loc.column,
            endLineNumber: e.loc.line,
            endColumn: e.loc.column,
            message: e.message
          }]);
        }
      }, 240);
    } else {
      submit();
      declarationRef.current = editorRef.current.deltaDecorations(declarationRef.current, [{
        range: new monacoRef.current.Range(1, 1, 1, 1),
        options: {}
      }]);
    }
  };
  var onChangeHandler = function onChangeHandler(value) {
    changedRef.current = true;
    valueRef.current = value;
    validate();
  };
  computedLanguage.current = language || defaultLanguage;
  realLanguage.current = /(?:javascript|typescript)/gi.test(computedLanguage.current) ? 'typescript' : computedLanguage.current;
  var renderHelpCode = function renderHelpCode() {
    if (!helpCode) return null;
    return /*#__PURE__*/React.createElement("div", {
      className: prefixCls + "-view",
      style: {
        width: helpCodeViewWidth || '50%'
      }
    }, /*#__PURE__*/React.createElement(Editor, {
      value: helpCode,
      theme: theme === 'dark' ? 'monokai' : 'chrome-devtools',
      defaultLanguage: realLanguage.current,
      language: realLanguage.current,
      options: _extends({}, props.options, {
        lineNumbers: 'off',
        readOnly: true,
        glyphMargin: false,
        folding: false,
        lineDecorationsWidth: 0,
        lineNumbersMinChars: 0,
        minimap: {
          enabled: false
        },
        tabSize: 2,
        smoothScrolling: true,
        scrollbar: {
          verticalScrollbarSize: 5,
          horizontalScrollbarSize: 5,
          alwaysConsumeMouseWheel: false
        }
      }),
      width: "100%",
      height: "100%"
    }));
  };
  return wrapSSR(/*#__PURE__*/React.createElement("div", {
    className: cls(prefixCls, hashId, className, {
      loaded: loaded
    }),
    style: {
      width: width,
      height: height
    }
  }, renderHelper(), /*#__PURE__*/React.createElement("div", {
    className: prefixCls + "-view"
  }, /*#__PURE__*/React.createElement(Editor, _extends({}, props, {
    theme: theme === 'dark' ? 'monokai' : 'chrome-devtools',
    defaultLanguage: realLanguage.current,
    language: realLanguage.current,
    options: _extends({
      // 语法标记边栏（用于错误提示）
      glyphMargin: true
    }, props.options, {
      tabSize: 2,
      // 平滑滚动
      smoothScrolling: true,
      // 滚动条设置
      scrollbar: {
        verticalScrollbarSize: 5,
        // 垂直滚动条宽度
        horizontalScrollbarSize: 5,
        // 水平滚动条高度
        alwaysConsumeMouseWheel: false // 是否始终响应鼠标滚轮
      },
      lineNumbersMinChars: 1
    }),
    value: input,
    width: "100%",
    height: "100%",
    onMount: onMountHandler
  }))), renderHelpCode()));
};
MonacoInput.loader = loader;