import _Dropdown from "antd/es/dropdown";
import _Button from "antd/es/button";
import _extends from "@babel/runtime/helpers/extends";
import { ArrayBase, ArrayItems } from '@formily/antd-v5';
import { usePrefixCls } from '@formily/antd-v5/lib/__builtins__';
import { Schema } from '@formily/json-schema';
import { observer, SchemaContext, useField, useFieldSchema } from '@formily/react';
import React, { forwardRef, useContext, useMemo } from 'react';
import cls from 'classnames';
import { useStyle } from './style';
import { isUndef } from '@formily/shared';
import { set } from 'lodash';
import { DeleteOutlined } from '@ant-design/icons';
import { SideBaseContext, useIndex } from '../array-advance-table/side-content';
var validatorFormats = [{
  label: 'URL地址',
  value: 'url'
}, {
  label: '邮箱格式',
  value: 'email'
}, {
  label: '数字格式',
  value: 'number'
}, {
  label: '整数格式',
  value: 'integer'
}, {
  label: '小数格式',
  value: 'float'
}, {
  label: '身份证格式',
  value: 'idcard'
}, {
  label: '手机号格式',
  value: 'phone'
}, {
  label: '货币格式',
  value: 'money'
}, {
  label: '中文格式',
  value: 'zh'
}, {
  label: '日期格式',
  value: 'date'
}, {
  label: '邮编格式',
  value: 'zip'
}];
var validatorSchema = {
  type: 'array',
  items: {
    type: 'object',
    'x-component': 'FormLayout',
    'x-component-props': {
      feedbackLayout: 'terse',
      labelCol: 6,
      colon: false,
      labelAlign: 'left'
    },
    properties: {
      group: {
        type: 'void',
        'x-component': 'RuleGroup',
        properties: {
          layout: {
            type: 'void',
            'x-component': 'FormRow',
            properties: {
              col1: {
                type: 'void',
                'x-component': 'FormRow.Col',
                'x-component-props': {
                  flex: '25px',
                  style: {
                    display: 'flex',
                    alignItems: 'center'
                  }
                },
                properties: {
                  sort: {
                    type: 'void',
                    'x-decorator': 'FormItem',
                    'x-component': 'ArrayItems.SortHandle'
                  }
                }
              },
              col2: {
                type: 'void',
                'x-component': 'FormRow.Col',
                'x-component-props': {
                  flex: 'auto'
                },
                properties: {
                  __ruleType__: {
                    type: 'string',
                    'x-hidden': true
                  },
                  requiredGroup: {
                    type: 'void',
                    'x-hidden': '{{ !($self.query(".__ruleType__").value() == "required") }}',
                    properties: {
                      required: {
                        type: 'string',
                        title: '必填',
                        'x-decorator': 'FormItem',
                        'x-component': 'Switch',
                        'x-hidden': true
                      },
                      message: {
                        type: 'string',
                        title: '必填',
                        'x-decorator': 'FormItem',
                        'x-component': 'Input'
                      }
                    }
                  },
                  patternGroup: {
                    type: 'void',
                    'x-hidden': '{{ !($self.query(".__ruleType__").value() === "pattern") }}',
                    properties: {
                      pattern: {
                        type: 'string',
                        title: '正则',
                        'x-decorator': 'FormItem',
                        'x-component': 'Input',
                        'x-component-props': {
                          prefix: '/',
                          suffix: '/'
                        }
                      },
                      message: {
                        type: 'string',
                        title: '提示信息',
                        'x-decorator': 'FormItem',
                        'x-component': 'Input'
                      }
                    }
                  },
                  formatGroup: {
                    type: 'void',
                    'x-hidden': '{{ !($self.query(".__ruleType__").value() === "format") }}',
                    properties: {
                      format: {
                        type: 'string',
                        title: '格式',
                        "enum": validatorFormats,
                        'x-decorator': 'FormItem',
                        'x-component': 'Select',
                        'x-component-props': {
                          allowClear: true
                        }
                      },
                      message: {
                        type: 'string',
                        title: '提示信息',
                        'x-decorator': 'FormItem',
                        'x-component': 'Input'
                      }
                    }
                  },
                  lenGroup: {
                    type: 'void',
                    'x-hidden': '{{ !($self.query(".__ruleType__").value() === "len") }}',
                    properties: {
                      len: {
                        type: 'string',
                        title: '长度',
                        'x-decorator': 'FormItem',
                        'x-component': 'InputNumber'
                      },
                      message: {
                        type: 'string',
                        title: '提示信息',
                        'x-decorator': 'FormItem',
                        'x-component': 'Input'
                      }
                    }
                  },
                  minGroup: {
                    type: 'void',
                    'x-hidden': '{{ !($self.query(".__ruleType__").value() === "min") }}',
                    properties: {
                      min: {
                        type: 'string',
                        title: '最小值',
                        'x-decorator': 'FormItem',
                        'x-component': 'InputNumber'
                      },
                      message: {
                        type: 'string',
                        title: '提示信息',
                        'x-decorator': 'FormItem',
                        'x-component': 'Input'
                      }
                    }
                  },
                  maxGroup: {
                    type: 'void',
                    'x-hidden': '{{ !($self.query(".__ruleType__").value() === "max") }}',
                    properties: {
                      max: {
                        type: 'string',
                        title: '最大值',
                        'x-decorator': 'FormItem',
                        'x-component': 'InputNumber'
                      },
                      message: {
                        type: 'string',
                        title: '提示信息',
                        'x-decorator': 'FormItem',
                        'x-component': 'Input'
                      }
                    }
                  },
                  dateGroup: {
                    type: 'void',
                    'x-hidden': '{{ !($self.query(".__ruleType__").value() === "date") }}',
                    properties: {
                      date: {
                        type: 'string',
                        title: '日期',
                        'x-decorator': 'FormItem',
                        'x-component': 'Select',
                        'x-component-props': {
                          allowClear: true
                        }
                      },
                      message: {
                        type: 'string',
                        title: '提示信息',
                        'x-decorator': 'FormItem',
                        'x-component': 'Input'
                      }
                    }
                  },
                  whitespaceGroup: {
                    type: 'void',
                    'x-hidden': '{{ !($self.query(".__ruleType__").value() === "whitespace") }}',
                    properties: {
                      whitespace: {
                        type: 'string',
                        title: '空格',
                        'x-decorator': 'FormItem',
                        'x-component': 'Switch'
                      },
                      message: {
                        type: 'string',
                        title: '提示信息'
                      }
                    }
                  }
                }
              },
              col3: {
                type: 'void',
                'x-component': 'FormRow.Col',
                'x-component-props': {
                  flex: '25px',
                  style: {
                    display: 'flex',
                    alignItems: 'center'
                  }
                },
                properties: {
                  remove: {
                    type: 'void',
                    'x-decorator': 'FormItem',
                    'x-component': 'ArrayItems.Remove'
                  }
                }
              }
            }
          }
        }
      }
    }
  },
  properties: {
    addValidatorRules: {
      type: 'void',
      title: '添加验证项',
      'x-component': 'ArrayRuleItems.Addition'
    }
  }
};
export var ValidatorSetter = observer(function () {
  var schema = useFieldSchema();
  var fieldSchema = useMemo(function () {
    return new Schema(validatorSchema, schema);
  }, []);
  return /*#__PURE__*/React.createElement(SchemaContext.Provider, {
    value: fieldSchema
  }, /*#__PURE__*/React.createElement(ArrayRuleItems, null));
});
var ruleTypes = [{
  key: 'required',
  label: '必填'
}, {
  key: 'pattern',
  label: '正则'
}, {
  key: 'format',
  label: '格式'
}, {
  key: 'len',
  label: '长度'
}, {
  key: 'min',
  label: '最小值'
}, {
  key: 'max',
  label: '最大值'
}, {
  key: 'date',
  label: '日期'
}];
var RuleAddition = observer(function (props) {
  // 需要限制只能添加一次的规则类型
  var uniqueRules = ['required', 'format', 'len', 'min', 'max', 'date'];
  var prefixCls = usePrefixCls('ant-formily-array-items');
  var _useStyle = useStyle(prefixCls),
    wrapSSR = _useStyle[0],
    hashId = _useStyle[1];
  var self = useField();
  var array = ArrayBase.useArray();
  var value = array.field.value || [];
  // 获取已存在的规则类型
  var existingRules = value.map(function (item) {
    return item.__ruleType__;
  });
  var items = ruleTypes.filter(function (item) {
    // 过滤已存在的唯一性规则
    return !(uniqueRules.includes(item.key) && existingRules.includes(item.key));
  });
  var onClick = function onClick(_ref) {
    var key = _ref.key,
      domEvent = _ref.domEvent;
    var defaultValue = {
      __ruleType__: key
    };
    if (key === 'required') {
      defaultValue = _extends({}, defaultValue, {
        required: true,
        message: '${name}必填'
      });
    }
    if (props.method === 'unshift') {
      var _array$field, _array$field$unshift, _array$props, _array$props$onAdd;
      (_array$field = array.field) === null || _array$field === void 0 ? void 0 : (_array$field$unshift = _array$field.unshift) === null || _array$field$unshift === void 0 ? void 0 : _array$field$unshift.call(_array$field, defaultValue);
      (_array$props = array.props) === null || _array$props === void 0 ? void 0 : (_array$props$onAdd = _array$props.onAdd) === null || _array$props$onAdd === void 0 ? void 0 : _array$props$onAdd.call(_array$props, 0);
    } else {
      var _array$field2, _array$field2$push, _array$props2, _array$props2$onAdd, _array$field3, _array$field3$value;
      (_array$field2 = array.field) === null || _array$field2 === void 0 ? void 0 : (_array$field2$push = _array$field2.push) === null || _array$field2$push === void 0 ? void 0 : _array$field2$push.call(_array$field2, defaultValue);
      (_array$props2 = array.props) === null || _array$props2 === void 0 ? void 0 : (_array$props2$onAdd = _array$props2.onAdd) === null || _array$props2$onAdd === void 0 ? void 0 : _array$props2$onAdd.call(_array$props2, (array === null || array === void 0 ? void 0 : (_array$field3 = array.field) === null || _array$field3 === void 0 ? void 0 : (_array$field3$value = _array$field3.value) === null || _array$field3$value === void 0 ? void 0 : _array$field3$value.length) - 1);
    }
    if (props.onClick) {
      props.onClick(domEvent);
    }
  };
  return wrapSSR(/*#__PURE__*/React.createElement("div", {
    className: cls(prefixCls + "-toobar", hashId, props.className)
  }, /*#__PURE__*/React.createElement(_Dropdown, {
    menu: {
      items: items,
      onClick: onClick
    },
    trigger: ['click']
  }, /*#__PURE__*/React.createElement(_Button, {
    color: "primary",
    variant: "filled"
  }, props.title || self.title)), /*#__PURE__*/React.createElement(Remove, null)));
});
var InternalArrayRuleItems = observer(function (props) {
  return /*#__PURE__*/React.createElement(ArrayItems, props);
});
export var ArrayRuleItems = Object.assign(InternalArrayRuleItems, {
  Addition: RuleAddition
});
var Remove = /*#__PURE__*/forwardRef(function (props, ref) {
  var _array$field4;
  var index = useIndex(props.index);
  var rootArray = useContext(SideBaseContext);
  var self = useField();
  var array = ArrayBase.useArray();
  var value = rootArray.field.value || [];
  var prefixCls = usePrefixCls('ant-formily-array-items');
  var _useStyle2 = useStyle(prefixCls),
    wrapSSR = _useStyle2[0],
    hashId = _useStyle2[1];
  if (!array) return null;
  if (((_array$field4 = array.field) === null || _array$field4 === void 0 ? void 0 : _array$field4.pattern) !== 'editable') return null;
  return wrapSSR(/*#__PURE__*/React.createElement(_Button, _extends({
    type: "ghost"
  }, props, {
    style: _extends({
      padding: 8,
      width: 'auto',
      height: 'auto'
    }, props.style),
    disabled: self === null || self === void 0 ? void 0 : self.disabled,
    className: cls(prefixCls + "-remove", hashId, self !== null && self !== void 0 && self.disabled ? prefixCls + "-remove-disabled" : '', props.className),
    ref: ref,
    onClick: function onClick(e) {
      var _rootArray$props, _rootArray$props$onRe;
      if (self !== null && self !== void 0 && self.disabled) return;
      e.stopPropagation();
      if (props.onClick) {
        props.onClick(e);
        if (e.defaultPrevented) return;
      }
      var path = array.field.path.slice(1).toString();
      set(value, path, undefined);
      (_rootArray$props = rootArray.props) === null || _rootArray$props === void 0 ? void 0 : (_rootArray$props$onRe = _rootArray$props.onRemove) === null || _rootArray$props$onRe === void 0 ? void 0 : _rootArray$props$onRe.call(_rootArray$props, index);
    },
    icon: isUndef(props.icon) ? /*#__PURE__*/React.createElement(DeleteOutlined, null) : props.icon
  }), props.title));
});