import _Tabs from "antd/es/tabs";
import _Breadcrumb from "antd/es/breadcrumb";
import _asyncToGenerator from "@babel/runtime/helpers/asyncToGenerator";
import _regeneratorRuntime from "@babel/runtime/regenerator";
import React, { useCallback, useContext, useEffect, useMemo, useState } from 'react';
import { Title, observer, globalContext, engineConfig } from '@gant-lowcode/lowcode-editor-core';
import { isSettingField } from '@gant-lowcode/lowcode-designer';
import classNames from 'classnames';
import { SettingsMain } from './main';
import { SettingsPane } from './settings-pane';
import { StageBox } from '../stage-box';
import { SkeletonContext } from '../../context';
import { intl } from '../../locale';
import { createIcon } from '@gant-lowcode/lowcode-utils';
import { SettingsForm } from './settings-from';
import { RenameNodeForm } from './components/rename-node-title';
import { EventContext, CanvasContext, CommonUIContext, EditorContext, SelectedContext, SettingsMainContext, useSettingsMain } from '../../hooks';
export var SettingsPrimaryPane = observer(function (_ref) {
  var _settings$getNode;
  var engineEditor = _ref.engineEditor,
    commonUI = _ref.commonUI,
    event = _ref.event,
    canvas = _ref.canvas,
    config = _ref.config;
  var _useState = useState(false),
    shouldIgnoreRoot = _useState[0],
    setShouldIgnoreRoot = _useState[1];
  var _useState2 = useState(),
    activeKey = _useState2[0],
    setActiveKey = _useState2[1];
  var main = useMemo(function () {
    return new SettingsMain(engineEditor);
  }, [engineEditor]);
  var skeleton = useContext(SkeletonContext);
  var setShouldIgnoreRootAsync = useCallback(/*#__PURE__*/_asyncToGenerator(/*#__PURE__*/_regeneratorRuntime.mark(function _callee() {
    var designMode;
    return _regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) switch (_context.prev = _context.next) {
        case 0:
          _context.next = 2;
          return globalContext.get('editor').get('designMode');
        case 2:
          designMode = _context.sent;
          setShouldIgnoreRoot(designMode === 'live');
        case 4:
        case "end":
          return _context.stop();
      }
    }, _callee);
  })), []);
  useEffect(function () {
    setShouldIgnoreRootAsync();
    var handleSelectionChange = function handleSelectionChange() {
      if (!engineConfig.get('stayOnTheSameSettingTab', false)) {
        setActiveKey(undefined);
      }
    };
    engineEditor.eventBus.on('designer.selection.change', handleSelectionChange);
    return function () {
      engineEditor.eventBus.off('designer.selection.change', handleSelectionChange);
      main.purge();
    };
  }, [engineEditor, main, setShouldIgnoreRootAsync]);
  var settings = main.settings;
  console.log('--settings', settings);
  if (!settings) {
    // 未选中节点，提示选中 或者 显示根节点设置
    return /*#__PURE__*/React.createElement("div", {
      className: "lc-settings-main"
    }, /*#__PURE__*/React.createElement("div", {
      className: "lc-settings-notice"
    }, /*#__PURE__*/React.createElement("p", null, intl('Please select a node in canvas'))));
  }

  // 当节点被锁定，且未开启锁定后容器可设置属性
  if (settings.isLocked && !engineConfig.get('enableLockedNodeSetting', false)) {
    return /*#__PURE__*/React.createElement("div", {
      className: "lc-settings-main"
    }, /*#__PURE__*/React.createElement("div", {
      className: "lc-settings-notice"
    }, /*#__PURE__*/React.createElement("p", null, intl('Current node is locked'))));
  }
  if (Array.isArray(settings.items) && settings.items.length === 0) {
    return /*#__PURE__*/React.createElement("div", {
      className: "lc-settings-main"
    }, /*#__PURE__*/React.createElement("div", {
      className: "lc-settings-notice"
    }, /*#__PURE__*/React.createElement("p", null, intl('No config found for this type of component'))));
  }
  if (!settings.isSameComponent) {
    // TODO: future support 获取设置项交集编辑
    return /*#__PURE__*/React.createElement("div", {
      className: "lc-settings-main"
    }, /*#__PURE__*/React.createElement("div", {
      className: "lc-settings-notice"
    }, /*#__PURE__*/React.createElement("p", null, intl('Please select same kind of components'))));
  }
  var renderBreadcrumb = function renderBreadcrumb() {};
  var items = settings.items;
  if (items.length > 5 || items.some(function (item) {
    return !isSettingField(item) || !item.isGroup;
  })) {
    return /*#__PURE__*/React.createElement("div", {
      className: "lc-settings-main"
    }, renderBreadcrumb(), /*#__PURE__*/React.createElement("div", {
      className: "lc-settings-body"
    }, /*#__PURE__*/React.createElement(StageBox, {
      skeleton: skeleton,
      target: settings,
      key: settings.id
    }, /*#__PURE__*/React.createElement(SettingsPane, {
      target: settings,
      usePopup: false
    }))));
  }
  var className = classNames('lc-settings-main', {
    'lc-settings-hide-tabs': items.length === 1 && engineConfig.get('hideSettingsTabsWhenOnlyOneItem', false)
  });
  var selected = settings === null || settings === void 0 ? void 0 : (_settings$getNode = settings.getNode()) === null || _settings$getNode === void 0 ? void 0 : _settings$getNode.internalToShellNode();
  console.log('---selected', selected);
  return /*#__PURE__*/React.createElement(EventContext.Provider, {
    value: event
  }, /*#__PURE__*/React.createElement(CanvasContext.Provider, {
    value: canvas
  }, /*#__PURE__*/React.createElement(CommonUIContext.Provider, {
    value: commonUI
  }, /*#__PURE__*/React.createElement(EditorContext.Provider, {
    value: engineEditor
  }, /*#__PURE__*/React.createElement(SettingsMainContext.Provider, {
    value: main
  }, /*#__PURE__*/React.createElement(SelectedContext.Provider, {
    value: selected
  }, /*#__PURE__*/React.createElement("div", {
    className: className
  }, /*#__PURE__*/React.createElement(SettingsBreadcrumb, {
    shouldIgnoreRoot: shouldIgnoreRoot,
    main: main
  }), /*#__PURE__*/React.createElement(Settings, {
    activeKey: activeKey,
    main: main,
    settings: settings
  }))))))));
});
var SettingsBreadcrumb = function SettingsBreadcrumb(_ref3) {
  var _designer$currentSele, _designer$currentSele2, _node$document, _main$componentMeta;
  var shouldIgnoreRoot = _ref3.shouldIgnoreRoot,
    main = _ref3.main;
  var settings = main.settings,
    editor = main.editor;
  if (!settings) {
    return null;
  }
  if (settings.isMultiple) {
    var _settings$componentMe;
    return /*#__PURE__*/React.createElement("div", {
      className: "lc-settings-navigator"
    }, createIcon((_settings$componentMe = settings.componentMeta) === null || _settings$componentMe === void 0 ? void 0 : _settings$componentMe.icon, {
      className: 'lc-settings-navigator-icon'
    }), /*#__PURE__*/React.createElement("div", {
      style: {
        marginLeft: '5px'
      }
    }, /*#__PURE__*/React.createElement(Title, {
      title: settings.componentMeta.title
    }), /*#__PURE__*/React.createElement("span", null, " x ", settings.nodes.length)));
  }
  var designer = editor.get('designer');
  var current = designer === null || designer === void 0 ? void 0 : (_designer$currentSele = designer.currentSelection) === null || _designer$currentSele === void 0 ? void 0 : (_designer$currentSele2 = _designer$currentSele.getNodes()) === null || _designer$currentSele2 === void 0 ? void 0 : _designer$currentSele2[0];
  var node = settings.first;
  var focusNode = (_node$document = node.document) === null || _node$document === void 0 ? void 0 : _node$document.focusNode;
  var items = [];
  var l = 3;
  var _loop = function _loop() {
    var _node = node;
    // dirty code: should remove
    if (shouldIgnoreRoot && node.isRoot()) {
      return 1; // break
    }
    if (focusNode && node.contains(focusNode)) {
      l = 0;
    }
    var item = l === 2 ? {
      title: /*#__PURE__*/React.createElement(Title, {
        title: node.title
      })
    } : {
      onMouseOver: hoverNode.bind(null, _node, true),
      onMouseOut: hoverNode.bind(null, _node, false),
      onClick: function onClick() {
        if (!_node) {
          return;
        }
        selectNode.call(null, _node);
        var getName = function getName(node) {
          var _node$componentMeta, _node$componentMeta2;
          var npm = node === null || node === void 0 ? void 0 : (_node$componentMeta = node.componentMeta) === null || _node$componentMeta === void 0 ? void 0 : _node$componentMeta.npm;
          return [npm === null || npm === void 0 ? void 0 : npm["package"], npm === null || npm === void 0 ? void 0 : npm.componentName].filter(function (item) {
            return !!item;
          }).join('-') || (node === null || node === void 0 ? void 0 : (_node$componentMeta2 = node.componentMeta) === null || _node$componentMeta2 === void 0 ? void 0 : _node$componentMeta2.componentName) || '';
        };
        var selected = getName(current);
        var target = getName(_node);
        editor === null || editor === void 0 ? void 0 : editor.eventBus.emit('skeleton.settingsPane.Breadcrumb', {
          selected: selected,
          target: target
        });
      },
      title: /*#__PURE__*/React.createElement(Title, {
        title: node.title
      })
    };
    items.unshift(item);
    node = node.parent;
  };
  while (l-- > 0 && node) {
    if (_loop()) break;
  }
  return /*#__PURE__*/React.createElement("div", {
    className: "lc-settings-navigator"
  }, createIcon((_main$componentMeta = main.componentMeta) === null || _main$componentMeta === void 0 ? void 0 : _main$componentMeta.icon, {
    className: 'lc-settings-navigator-icon'
  }), /*#__PURE__*/React.createElement(_Breadcrumb, {
    className: "lc-settings-node-breadcrumb",
    separator: ">",
    items: items
  }));
};
export var Settings = function Settings(_ref4) {
  var _settings$getNode2, _items$;
  var settings = _ref4.settings,
    _activeKey = _ref4.activeKey;
  var main = useSettingsMain();
  var _useState3 = useState(_activeKey),
    activeKey = _useState3[0],
    setActiveKey = _useState3[1];
  var skeleton = useContext(SkeletonContext);
  var handleTabChange = useCallback(function (key) {
    setActiveKey(key);
  }, []);
  var items = settings.items;
  var handleTabClick = useCallback(function (key) {
    var field = items.find(function (item) {
      return item.name === key;
    });
    main.skeletonSettingsPaneChange({
      name: field.name,
      title: field.title
    });
  }, [items]);
  console.log('---items', main, settings, items);
  var tabs = items.map(function (field) {
    return {
      key: String(field.name),
      label: /*#__PURE__*/React.createElement(Title, {
        title: field.title
      }),
      children: field.schema ? /*#__PURE__*/React.createElement(React.Fragment, null, field.name === '#props' && /*#__PURE__*/React.createElement(RenameNodeForm, {
        key: "rename-" + field.id,
        main: main
      }), /*#__PURE__*/React.createElement(SettingsForm, {
        key: "props-" + field.id,
        SchemaField: main.SchemaField,
        settings: settings,
        schema: field.schema
      })) : /*#__PURE__*/React.createElement(StageBox, {
        skeleton: skeleton,
        target: field,
        key: field.id
      }, /*#__PURE__*/React.createElement(SettingsPane, {
        target: field,
        key: field.id,
        usePopup: false
      }))
    };
  });
  var selected = settings === null || settings === void 0 ? void 0 : (_settings$getNode2 = settings.getNode()) === null || _settings$getNode2 === void 0 ? void 0 : _settings$getNode2.internalToShellNode();
  console.log('---selected', selected);
  return /*#__PURE__*/React.createElement(_Tabs, {
    className: "lc-settings-tabs",
    defaultActiveKey: (_items$ = items[0]) === null || _items$ === void 0 ? void 0 : _items$.name,
    activeKey: activeKey,
    items: tabs,
    indicator: {
      align: 'center'
    },
    onChange: handleTabChange,
    onTabClick: handleTabClick
  });
};
function hoverNode(node, flag) {
  node.hover(flag);
}
function selectNode(node) {
  node === null || node === void 0 ? void 0 : node.select();
}