import React, { useEffect, useRef, useState } from 'react';
import classNames from 'classnames';
import { observer } from '@gant-lowcode/lowcode-editor-core';
import StageChain from './stage-chain';
import Stage from './stage';
import PopupService, { PopupPipe } from '../popup';
export var StageBox = observer(function (props) {
  function createDefaultChain() {
    var stateName = props.skeleton.createStage({
      content: props.children,
      isRoot: true
    });
    return new StageChain(props.skeleton.getStage(stateName));
  }
  var _useState = useState(props.stageChain || createDefaultChain()),
    stageChain = _useState[0];
  var _useState2 = useState({}),
    _ = _useState2[0],
    forceUpdate = _useState2[1];
  var shellRef = useRef(null);
  var popupPipe = useRef(new PopupPipe());
  useEffect(function () {
    var unsubscribe = stageChain.onStageChange(function () {
      return forceUpdate({});
    });
    return function () {
      return unsubscribe();
    };
  }, [stageChain]);
  useEffect(function () {
    if (!shellRef.current) {
      return;
    }
    var shell = shellRef.current;

    /**
     * 向上层递归寻找 target
     * @param node 节点
     * @returns 节点的 dataset.stageTarget 信息
     */
    var _getTarget = function getTarget(node) {
      if (!node || !shell.contains(node) || node.nodeName === 'A' && node.getAttribute('href')) {
        return null;
      }
      var target = node.dataset ? node.dataset.stageTarget : null;
      if (target) {
        return target;
      }
      return _getTarget(node.parentNode);
    };
    var clickHandler = function clickHandler(e) {
      var target = _getTarget(e.target);
      if (!target) {
        return;
      }
      if (target === 'stageback') {
        stageChain.stageBack();
      } else if (target === 'stageexit') {
        stageChain.stageBackToRoot();
      } else {
        stageChain.stagePush(props.skeleton.getStage(target));
      }
    };
    shell.addEventListener('click', clickHandler, false);
    return function () {
      shell.removeEventListener('click', clickHandler);
    };
  }, [props.skeleton, stageChain]);
  var currentStage = stageChain.getCurrentStage();
  var refer = currentStage === null || currentStage === void 0 ? void 0 : currentStage.getRefer();
  return /*#__PURE__*/React.createElement("div", {
    ref: shellRef,
    className: classNames('skeleton-stagebox', props.className)
  }, /*#__PURE__*/React.createElement(PopupService, {
    popupPipe: popupPipe.current
  }, refer && /*#__PURE__*/React.createElement(Stage, {
    key: refer.stage.getId(),
    stage: refer.stage,
    direction: refer.direction
  }), /*#__PURE__*/React.createElement(Stage, {
    key: currentStage.getId(),
    stage: currentStage,
    direction: refer === null || refer === void 0 ? void 0 : refer.direction,
    current: true
  })));
});