import _ConfigProvider from "antd/es/config-provider";
import { useState, useEffect } from 'react';
import { TipContainer, observer } from '@gant-lowcode/lowcode-editor-core';
import classNames from 'classnames';
import TopArea from './top-area';
import LeftArea from './left-area';
import LeftFixedPane from './left-fixed-pane';
import LeftFloatPane from './left-float-pane';
import Toolbar from './toolbar';
import MainArea from './main-area';
import BottomArea from './bottom-area';
import RightArea from './right-area';
import './workbench.less';
import { SkeletonContext } from '../context';
import zhCN from 'antd/locale/zh_CN';
var skeletonTheme = {
  token: {
    colorTextHeading: '#666',
    fontSize: 12 // 全局基础字号
  },
  components: {
    Card: {
      bodyPadding: 12,
      headerPadding: 12
    }
  }
};
export var Workbench = observer(function (_ref) {
  var skeleton = _ref.skeleton,
    config = _ref.config,
    components = _ref.components,
    className = _ref.className,
    topAreaItemClassName = _ref.topAreaItemClassName;
  var _useState = useState(400),
    width = _useState[0],
    setWidth = _useState[1];
  var _useState2 = useState(false),
    startDrag = _useState2[0],
    setStartDrag = _useState2[1];
  useEffect(function () {
    skeleton.buildFromConfig(config, components);
  }, []);
  var handleSplitterMouseDown = function handleSplitterMouseDown(e) {
    e.preventDefault();
    var startX = e.clientX;
    var startWidth = width;
    setStartDrag(true);
    var doDrag = function doDrag(moveEvent) {
      var deltaX = moveEvent.clientX - startX;
      var newWidth = startWidth - deltaX;
      setWidth(Math.max(400, Math.min(800, newWidth)));
    };
    var _stopDrag = function stopDrag() {
      setStartDrag(false);
      document.removeEventListener('mousemove', doDrag);
      document.removeEventListener('mouseup', _stopDrag);
    };
    document.addEventListener('mousemove', doDrag);
    document.addEventListener('mouseup', _stopDrag);
  };
  return /*#__PURE__*/React.createElement("div", {
    className: classNames('lc-workbench', className)
  }, /*#__PURE__*/React.createElement(_ConfigProvider, {
    theme: skeletonTheme,
    locale: zhCN
  }, /*#__PURE__*/React.createElement(SkeletonContext.Provider, {
    value: skeleton
  }, /*#__PURE__*/React.createElement("div", {
    className: "lc-workbench-body"
  }, /*#__PURE__*/React.createElement(LeftArea, {
    area: skeleton.leftArea
  }), /*#__PURE__*/React.createElement(LeftFloatPane, {
    area: skeleton.leftFloatArea
  }), /*#__PURE__*/React.createElement(LeftFixedPane, {
    area: skeleton.leftFixedArea
  }), /*#__PURE__*/React.createElement("div", {
    className: "lc-workbench-body-right"
  }, /*#__PURE__*/React.createElement(TopArea, {
    area: skeleton.topArea,
    itemClassName: topAreaItemClassName
  }), /*#__PURE__*/React.createElement("div", {
    className: "lc-workbench-splitter"
  }, /*#__PURE__*/React.createElement("div", {
    className: "lc-workbench-center left-area-splitter-panel"
  }, /*#__PURE__*/React.createElement(Toolbar, {
    area: skeleton.toolbar
  }), /*#__PURE__*/React.createElement(MainArea, {
    area: skeleton.mainArea
  }), /*#__PURE__*/React.createElement(BottomArea, {
    area: skeleton.bottomArea
  })), /*#__PURE__*/React.createElement("div", {
    className: "splitter-bar",
    onMouseDown: handleSplitterMouseDown
  }, /*#__PURE__*/React.createElement("div", {
    className: "splitter-bar-dragger"
  })), /*#__PURE__*/React.createElement("div", {
    className: "right-area-splitter-panel",
    style: {
      width: width
    }
  }, /*#__PURE__*/React.createElement(RightArea, {
    area: skeleton.rightArea
  })), startDrag && /*#__PURE__*/React.createElement("div", {
    "aria-hidden": true,
    className: "splitter-mask"
  })))), /*#__PURE__*/React.createElement(TipContainer, null))));
});