"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
exports.__esModule = true;
exports.FormSwitchObject = void 0;
var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));
var _switch = _interopRequireDefault(require("antd/lib/switch"));
var _objectWithoutPropertiesLoose2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutPropertiesLoose"));
var _react = _interopRequireDefault(require("react"));
var _react2 = require("@formily/react");
var _antdV = require("@formily/antd-v5");
var _classnames = _interopRequireDefault(require("classnames"));
var _style = _interopRequireDefault(require("./style"));
var _hooks = require("../../../../__builtins__/hooks");
var _excluded = ["className", "style", "title", "children"];
/**
 * Schema 驱动的开关对象字段组件
 *
 * 在 schema 中使用：
 * ```json
 * inputSearchProps: {
 *   type: 'object',
 *   'x-decorator': 'FormItem',
 *   'x-component': 'SwitchObjectField',
 *   'x-component-props': { title: '简易搜索' },
 *   properties: { ... 子字段配置 ... }
 * }
 * ```
 *
 * 当开关关闭时，字段值为 undefined；
 * 当开关开启时，字段值为包含子属性的对象。
 */
var FormSwitchObject = exports.FormSwitchObject = (0, _react2.observer)(function (_ref) {
  var className = _ref.className,
    style = _ref.style,
    title = _ref.title,
    children = _ref.children,
    restProps = (0, _objectWithoutPropertiesLoose2["default"])(_ref, _excluded);
  var field = (0, _react2.useField)();
  console.log('-------', field, '-------', restProps);
  var value = field.value;
  var enabled = !!value;
  var prefixCls = (0, _hooks.usePrefixCls)('form-switch-object', restProps);
  var _useStyle = (0, _style["default"])(prefixCls),
    wrapSSR = _useStyle[0],
    hashId = _useStyle[1];
  // 切换开关：开启则初始化空对象，关闭则置为 undefined
  var handleToggle = function handleToggle(checked) {
    if (checked) {
      field.setValue({});
    } else {
      field.setValue(false);
    }
  };
  var deepLayout = (0, _antdV.useFormLayout)();
  console.log('---deepLayout', deepLayout);
  return wrapSSR(/*#__PURE__*/_react["default"].createElement("div", {
    className: (0, _classnames["default"])(prefixCls + "-layout", hashId, className),
    style: style
  }, /*#__PURE__*/_react["default"].createElement(_antdV.FormItem, (0, _extends2["default"])({
    label: (field === null || field === void 0 ? void 0 : field.title) || '启用'
  }, deepLayout), /*#__PURE__*/_react["default"].createElement(_switch["default"], {
    checked: enabled,
    onChange: handleToggle
  })), enabled && children));
});