"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
exports.__esModule = true;
exports.MonacoInput = void 0;
var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));
var _tooltip = _interopRequireDefault(require("antd/lib/tooltip"));
var _objectWithoutPropertiesLoose2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutPropertiesLoose"));
var _react = _interopRequireWildcard(require("react"));
var _react2 = require("@monaco-editor/react");
var _parser = require("@babel/parser");
var _format = require("./format");
var _classnames = _interopRequireDefault(require("classnames"));
var _config = require("./config");
var _shared = require("@formily/shared");
var _lowcodeUtils = require("@gant-lowcode/lowcode-utils");
var _builtins__ = require("@formily/antd-v5/lib/__builtins__");
var _style = require("./style");
var _excluded = ["className", "language", "defaultLanguage", "width", "helpLink", "helpCode", "helpCodeViewWidth", "height", "onMount", "onChange", "theme"];
function _getRequireWildcardCache(e) { if ("function" != typeof WeakMap) return null; var r = new WeakMap(), t = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(e) { return e ? t : r; })(e); }
function _interopRequireWildcard(e, r) { if (!r && e && e.__esModule) return e; if (null === e || "object" != typeof e && "function" != typeof e) return { "default": e }; var t = _getRequireWildcardCache(r); if (t && t.has(e)) return t.get(e); var n = { __proto__: null }, a = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var u in e) if ("default" !== u && {}.hasOwnProperty.call(e, u)) { var i = a ? Object.getOwnPropertyDescriptor(e, u) : null; i && (i.get || i.set) ? Object.defineProperty(n, u, i) : n[u] = e[u]; } return n["default"] = e, t && t.set(e, n), n; }
var MonacoInput = exports.MonacoInput = function MonacoInput(_ref) {
  var className = _ref.className,
    language = _ref.language,
    defaultLanguage = _ref.defaultLanguage,
    width = _ref.width,
    helpLink = _ref.helpLink,
    helpCode = _ref.helpCode,
    helpCodeViewWidth = _ref.helpCodeViewWidth,
    height = _ref.height,
    onMount = _ref.onMount,
    onChange = _ref.onChange,
    theme = _ref.theme,
    props = (0, _objectWithoutPropertiesLoose2["default"])(_ref, _excluded);
  var _useState = (0, _react.useState)(false),
    loaded = _useState[0],
    setLoaded = _useState[1];
  var valueRef = (0, _react.useRef)('');
  var validateRef = (0, _react.useRef)(null);
  var submitRef = (0, _react.useRef)(null);
  var declarationRef = (0, _react.useRef)([]);
  var extraLibRef = (0, _react.useRef)(null);
  var monacoRef = (0, _react.useRef)();
  var editorRef = (0, _react.useRef)();
  var computedLanguage = (0, _react.useRef)(language || defaultLanguage);
  var realLanguage = (0, _react.useRef)('');
  var unmountedRef = (0, _react.useRef)(false);
  var changedRef = (0, _react.useRef)(false);
  var uidRef = (0, _react.useRef)((0, _shared.uid)());
  var prefixCls = (0, _builtins__.usePrefixCls)('monaco-input', props);
  var _useStyle = (0, _style.useStyle)(prefixCls),
    wrapSSR = _useStyle[0],
    hashId = _useStyle[1];
  var input = props.value || props.defaultValue;
  (0, _react.useEffect)(function () {
    unmountedRef.current = false;
    (0, _config.initMonaco)();
    return function () {
      if (extraLibRef.current) {
        extraLibRef.current.dispose();
      }
      unmountedRef.current = true;
    };
  }, []);
  (0, _react.useEffect)(function () {
    if (monacoRef.current && props.extraLib) {
      updateExtraLib();
    }
  }, [props.extraLib]);
  var updateExtraLib = function updateExtraLib() {
    if (extraLibRef.current) {
      extraLibRef.current.dispose();
    }
    extraLibRef.current = monacoRef.current.languages.typescript.typescriptDefaults.addExtraLib(props.extraLib, uidRef.current + ".d.ts");
  };
  var isFileLanguage = function isFileLanguage() {
    var lang = computedLanguage.current;
    return lang === 'javascript' || lang === 'typescript';
  };
  var isExpLanguage = function isExpLanguage() {
    var lang = computedLanguage.current;
    return lang === 'javascript.expression' || lang === 'typescript.expression';
  };
  var renderHelper = function renderHelper() {
    var getHref = function getHref() {
      if (typeof helpLink === 'string') return helpLink;
      if (isFileLanguage()) {
        return 'https://developer.mozilla.org/zh-CN/docs/Web/JavaScript';
      }
      if (isExpLanguage()) {
        return 'https://developer.mozilla.org/zh-CN/docs/Web/JavaScript/Reference/Operators';
      }
    };
    if (helpLink === false) return null;
    var href = getHref();
    return href && /*#__PURE__*/_react["default"].createElement(_tooltip["default"], {
      title: "\u5E2E\u52A9\u6587\u6863"
    }, /*#__PURE__*/_react["default"].createElement("div", {
      className: prefixCls + "-helper"
    }, /*#__PURE__*/_react["default"].createElement("a", {
      target: "_blank",
      href: href,
      rel: "noreferrer"
    }, (0, _lowcodeUtils.createIcon)('wenhao'))));
  };
  var onMountHandler = function onMountHandler(editor, monaco) {
    editorRef.current = editor;
    monacoRef.current = monaco;
    onMount === null || onMount === void 0 ? void 0 : onMount(editor, monaco);
    var model = editor.getModel();
    var currentValue = editor.getValue();
    model['getDesignerLanguage'] = function () {
      return computedLanguage.current;
    };
    if (currentValue) {
      (0, _format.format)(computedLanguage.current, currentValue).then(function (content) {
        editor.setValue(content);
        setLoaded(true);
      })["catch"](function () {
        setLoaded(true);
      });
    } else {
      setLoaded(true);
    }
    if (props.extraLib) {
      updateExtraLib();
    }
    editor.onDidChangeModelContent(function () {
      onChangeHandler(editor.getValue());
    });
  };
  var submit = function submit() {
    clearTimeout(submitRef.current);
    submitRef.current = setTimeout(function () {
      onChange === null || onChange === void 0 ? void 0 : onChange(valueRef.current);
    }, 1000);
  };
  var validate = function validate() {
    if (realLanguage.current === 'typescript') {
      clearTimeout(validateRef.current);
      validateRef.current = setTimeout(function () {
        try {
          if (valueRef.current) {
            if (isFileLanguage()) {
              (0, _parser.parse)(valueRef.current, {
                sourceType: 'module',
                plugins: ['typescript', 'jsx']
              });
            } else if (isExpLanguage()) {
              (0, _parser.parseExpression)(valueRef.current, {
                plugins: ['typescript', 'jsx']
              });
            }
          }
          monacoRef.current.editor.setModelMarkers(editorRef.current.getModel(), computedLanguage.current, []);
          declarationRef.current = editorRef.current.deltaDecorations(declarationRef.current, [{
            range: new monacoRef.current.Range(1, 1, 1, 1),
            options: {}
          }]);
          submit();
        } catch (e) {
          declarationRef.current = editorRef.current.deltaDecorations(declarationRef.current, [{
            range: new monacoRef.current.Range(e.loc.line, e.loc.column, e.loc.line, e.loc.column),
            options: {
              isWholeLine: true,
              glyphMarginClassName: 'monaco-error-highline'
            }
          }]);
          monacoRef.current.editor.setModelMarkers(editorRef.current.getModel(), computedLanguage.current, [{
            code: '1003',
            severity: 8,
            startLineNumber: e.loc.line,
            startColumn: e.loc.column,
            endLineNumber: e.loc.line,
            endColumn: e.loc.column,
            message: e.message
          }]);
        }
      }, 240);
    } else {
      submit();
      declarationRef.current = editorRef.current.deltaDecorations(declarationRef.current, [{
        range: new monacoRef.current.Range(1, 1, 1, 1),
        options: {}
      }]);
    }
  };
  var onChangeHandler = function onChangeHandler(value) {
    changedRef.current = true;
    valueRef.current = value;
    validate();
  };
  computedLanguage.current = language || defaultLanguage;
  realLanguage.current = /(?:javascript|typescript)/gi.test(computedLanguage.current) ? 'typescript' : computedLanguage.current;
  var renderHelpCode = function renderHelpCode() {
    if (!helpCode) return null;
    return /*#__PURE__*/_react["default"].createElement("div", {
      className: prefixCls + "-view",
      style: {
        width: helpCodeViewWidth || '50%'
      }
    }, /*#__PURE__*/_react["default"].createElement(_react2.Editor, {
      value: helpCode,
      theme: theme === 'dark' ? 'monokai' : 'chrome-devtools',
      defaultLanguage: realLanguage.current,
      language: realLanguage.current,
      options: (0, _extends2["default"])({}, props.options, {
        lineNumbers: 'off',
        readOnly: true,
        glyphMargin: false,
        folding: false,
        lineDecorationsWidth: 0,
        lineNumbersMinChars: 0,
        minimap: {
          enabled: false
        },
        tabSize: 2,
        smoothScrolling: true,
        scrollbar: {
          verticalScrollbarSize: 5,
          horizontalScrollbarSize: 5,
          alwaysConsumeMouseWheel: false
        }
      }),
      width: "100%",
      height: "100%"
    }));
  };
  return wrapSSR(/*#__PURE__*/_react["default"].createElement("div", {
    className: (0, _classnames["default"])(prefixCls, hashId, className, {
      loaded: loaded
    }),
    style: {
      width: width,
      height: height
    }
  }, renderHelper(), /*#__PURE__*/_react["default"].createElement("div", {
    className: prefixCls + "-view"
  }, /*#__PURE__*/_react["default"].createElement(_react2.Editor, (0, _extends2["default"])({}, props, {
    theme: theme === 'dark' ? 'monokai' : 'chrome-devtools',
    defaultLanguage: realLanguage.current,
    language: realLanguage.current,
    options: (0, _extends2["default"])({
      // 语法标记边栏（用于错误提示）
      glyphMargin: true
    }, props.options, {
      tabSize: 2,
      // 平滑滚动
      smoothScrolling: true,
      // 滚动条设置
      scrollbar: {
        verticalScrollbarSize: 5,
        // 垂直滚动条宽度
        horizontalScrollbarSize: 5,
        // 水平滚动条高度
        alwaysConsumeMouseWheel: false // 是否始终响应鼠标滚轮
      },
      lineNumbersMinChars: 1
    }),
    value: input,
    width: "100%",
    height: "100%",
    onMount: onMountHandler
  }))), renderHelpCode()));
};
MonacoInput.loader = _react2.loader;