"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
exports.__esModule = true;
exports.SettingsMain = void 0;
var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));
var _card = _interopRequireDefault(require("antd/lib/card"));
var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));
var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));
var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));
var _react = require("@formily/react");
var _antdV = require("@formily/antd-v5");
var _lowcodeEditorCore = require("@gant-lowcode/lowcode-editor-core");
var _components = require("./components");
var _reactive = require("@formily/reactive");
var _FormGrid = _antdV.FormGrid;
function generateSessionId(nodes) {
  return nodes.map(function (node) {
    return node.id;
  }).sort().join(',');
}
var SettingsMain = exports.SettingsMain = /*#__PURE__*/function () {
  function SettingsMain(editor) {
    this.editor = editor;
    this.emitter = (0, _lowcodeEditorCore.createModuleEventBus)('SettingsMain');
    this._sessionId = '';
    this._settings = void 0;
    this.disposeListener = void 0;
    this.designer = void 0;
    this.SchemaField = void 0;
    (0, _lowcodeEditorCore.makeObservable)(this, {
      _settings: _lowcodeEditorCore.obx.ref,
      setup: _lowcodeEditorCore.action,
      length: _lowcodeEditorCore.computed,
      componentMeta: _lowcodeEditorCore.computed,
      settings: _lowcodeEditorCore.computed
    });
    this.init();
    this.SchemaField = this.createSchemaField();
  }
  var _proto = SettingsMain.prototype;
  _proto.init = /*#__PURE__*/function () {
    var _init = (0, _asyncToGenerator2["default"])(/*#__PURE__*/_regenerator["default"].mark(function _callee() {
      var _this = this;
      var setupSelection, designer;
      return _regenerator["default"].wrap(function _callee$(_context) {
        while (1) switch (_context.prev = _context.next) {
          case 0:
            setupSelection = function setupSelection(selection) {
              if (selection) {
                _this.setup(selection.getNodes());
              } else {
                _this.setup([]);
              }
            };
            this.editor.eventBus.on('designer.selection.change', setupSelection);
            this.disposeListener = function () {
              _this.editor.removeListener('designer.selection.change', setupSelection);
            };
            _context.next = 5;
            return this.editor.onceGot('designer');
          case 5:
            designer = _context.sent;
            this.designer = designer;
            setupSelection(designer.currentSelection);
          case 8:
          case "end":
            return _context.stop();
        }
      }, _callee, this);
    }));
    function init() {
      return _init.apply(this, arguments);
    }
    return init;
  }();
  _proto.setup = function setup(nodes) {
    // check nodes change
    var sessionId = generateSessionId(nodes);
    if (sessionId === this._sessionId) {
      return;
    }
    this._sessionId = sessionId;
    if (nodes.length < 1) {
      this._settings = undefined;
      return;
    }
    if (!this.designer) {
      this.designer = nodes[0].document.designer;
    }
    // 当节点只有一个时，复用 node 上挂载的 settingEntry，不会产生平行的两个实例，这样在整个系统中对
    // 某个节点操作的 SettingTopEntry 只有一个实例，后续的 getProp() 也会拿到相同的 SettingField 实例
    if (nodes.length === 1) {
      this._settings = nodes[0].settingEntry;
    } else {
      this._settings = this.designer.createSettingEntry(nodes);
    }
    console.log('--this._settings', nodes, this._settings);
  };
  _proto.skeletonSettingsPaneChange = function skeletonSettingsPaneChange(params) {
    this.editor.eventBus.emit('skeleton.settingsPane.change', params);
  };
  _proto.purge = function purge() {
    this.disposeListener();
    this.emitter.removeAllListeners();
  };
  _proto.createSchemaField = function createSchemaField() {
    var _this$editor;
    var setters = (_this$editor = this.editor) === null || _this$editor === void 0 ? void 0 : _this$editor.get('setters');
    var settersMap = setters === null || setters === void 0 ? void 0 : setters.getSettersMap();
    var customComponents = {};
    settersMap === null || settersMap === void 0 ? void 0 : settersMap.forEach(function (setter) {
      customComponents[setter.type] = setter.component;
    });
    return (0, _react.createSchemaField)({
      components: (0, _extends2["default"])({
        Input: _antdV.Input,
        InputNumber: _antdV.NumberPicker,
        Select: _antdV.Select,
        Switch: _antdV.Switch,
        Radio: _antdV.Radio,
        Checkbox: _antdV.Checkbox,
        Card: _card["default"],
        FormCard: _components.FormCard,
        FormRow: _components.FormRow,
        FormItem: _antdV.FormItem,
        FormLayout: _antdV.FormLayout,
        FormGrid: _FormGrid,
        FormCollapse: _antdV.FormCollapse,
        ArrayCollapse: _antdV.ArrayCollapse,
        ArrayItems: _antdV.ArrayItems,
        ArrayTable: _antdV.ArrayTable,
        Space: _antdV.Space,
        DrawerSetter: _components.DrawerSetter,
        FormSwitchObject: _components.FormSwitchObject,
        NodeChildrenArrayTable: _components.NodeChildrenArrayTable,
        ArrayAdvanceTable: _components.ArrayAdvanceTable,
        FormSlotSetter: _components.FormSlotSetter,
        ValidatorSetter: _components.ValidatorSetter,
        ArrayRuleItems: _components.ArrayRuleItems,
        RuleGroup: _components.RuleGroup,
        ExpressionSetter: _components.ExpressionSetter,
        MonacoInput: _components.MonacoInput,
        Text: _components.Text,
        SelectInput: _components.SelectInput,
        DatePicker: _antdV.DatePicker,
        MixedSetter: _components.MixedSetter
      }, customComponents),
      scope: {
        useDataSource: this.useDataSource.bind(this)
      }
    });
  };
  _proto.useDataSource = function useDataSource(key, options) {
    var _this$editor2;
    var setters = (_this$editor2 = this.editor) === null || _this$editor2 === void 0 ? void 0 : _this$editor2.get('setters');
    var dataSourceService = setters.dataSourceService;
    return function (field) {
      field.loading = true;
      try {
        dataSourceService.resolve(key, field, {
          cacheable: options === null || options === void 0 ? void 0 : options.cacheable,
          paramsBuilder: options === null || options === void 0 ? void 0 : options.paramsBuilder
        }).then(_reactive.action.bound(function (data) {
          field.dataSource = data;
          field.loading = false;
        }));
      } catch (error) {
        field.setFeedback({
          type: 'error',
          code: 'DATA_FETCH_FAILED',
          messages: [error.message]
        });
        field.loading = false;
      }
    };
  };
  return (0, _createClass2["default"])(SettingsMain, [{
    key: "length",
    get: function get() {
      var _this$_settings;
      return (_this$_settings = this._settings) === null || _this$_settings === void 0 ? void 0 : _this$_settings.nodes.length;
    }
  }, {
    key: "componentMeta",
    get: function get() {
      var _this$_settings2;
      return (_this$_settings2 = this._settings) === null || _this$_settings2 === void 0 ? void 0 : _this$_settings2.componentMeta;
    }
  }, {
    key: "settings",
    get: function get() {
      return this._settings;
    }
  }]);
}();