"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
exports.__esModule = true;
exports.ExpressionSetter = void 0;
var _row = _interopRequireDefault(require("antd/lib/row"));
var _tree = _interopRequireDefault(require("antd/lib/tree"));
var _col = _interopRequireDefault(require("antd/lib/col"));
var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));
var _segmented = _interopRequireDefault(require("antd/lib/segmented"));
var _react = require("@formily/react");
var _antdV = require("@formily/antd-v5");
var _components = require("../../components/settings/components");
var reservedKeywords = new Set(['break', 'case', 'catch', 'class', 'const', 'continue', 'debugger', 'default', 'delete', 'do', 'else', 'enum', 'export', 'extends', 'false', 'finally', 'for', 'function', 'if', 'import', 'in', 'instanceof', 'new', 'null', 'return', 'super', 'switch', 'this', 'throw', 'true', 'try', 'typeof', 'var', 'void', 'while', 'with', 'yield', 'let', 'static', 'await', 'implements', 'interface', 'package', 'private', 'protected', 'public', /** react 相关 */
'useState', 'useEffect', 'useContext', 'useReducer', 'useCallback', 'useMemo', 'useRef', 'useImperativeHandle', 'useLayoutEffect', 'useDebugValue', 'Fragment', 'StrictMode', 'Suspense', 'lazy', 'createContext', 'Context', 'Provider', 'Consumer', 'memo', 'forwardRef', 'createRef']);
var validateVariableName = function validateVariableName(value) {
  if (!value) return Promise.reject(new Error('请输入变量名'));
  if (/^\d/.test(value)) {
    return Promise.reject(new Error('变量名不能以数字开头'));
  }
  if (/[^a-zA-Z0-9_$]/.test(value)) {
    return Promise.reject(new Error('变量名只能包含字母、数字、下划线(_)或美元符号($)'));
  }

  // if (/[^\x00-\x7F]/.test(value)) {
  //   return Promise.reject(new Error('变量名建议使用英文、数字或符号，避免使用中文字符'));
  // }

  if (reservedKeywords.has(value)) {
    return Promise.reject(new Error("\"" + value + "\" \u662F\u4FDD\u7559\u5173\u952E\u5B57\uFF0C\u4E0D\u80FD\u4F5C\u4E3A\u53D8\u91CF\u540D"));
  }
  return Promise.resolve();
};
var SchemaField = (0, _react.createSchemaField)({
  components: {
    FormItem: _antdV.FormItem,
    FormGrid: _antdV.FormGrid,
    FormLayout: _antdV.FormLayout,
    Input: _antdV.Input,
    Switch: _antdV.Switch,
    MonacoInput: _components.MonacoInput,
    Segmented: _segmented["default"]
  }
});
var options = [{
  value: 'expression',
  label: '表达式'
}, {
  value: 'boolean',
  label: '布尔值'
}, {
  value: 'ternary',
  label: '三元表达式'
}
// {
//   value: 'logic',
//   label: '逻辑判断',
// },
// {
//   value: 'custom',
//   label: '自定义函数',
// },
];
var schema = {
  type: 'object',
  properties: {
    type: {
      type: 'string',
      'x-decorator': 'FormItem',
      'x-component': 'Segmented',
      'x-component-props': {
        options: options
      }
    },
    value: {
      type: 'string',
      'x-decorator': 'FormItem',
      'x-component': 'MonacoInput',
      'x-component-props': {
        language: 'javascript.expression',
        placeholder: '请输入表达式',
        helpLink: false,
        height: 200
      }
    },
    layout: {
      type: 'void',
      'x-component': 'FormGrid',
      'x-component-props': {
        maxColumns: 2,
        minColumns: 2
      },
      properties: {
        isFixed: {
          type: 'string',
          title: '作为页面变量',
          'x-decorator': 'FormItem',
          'x-component': 'Switch'
        },
        variableName: {
          type: 'string',
          title: '变量名',
          'x-decorator': 'FormItem',
          'x-decorator-props': {
            gridSpan: 2
          },
          'x-component': 'Input',
          'x-component-props': {
            placeholder: '变量名'
          },
          'x-validator': [validateVariableName],
          'x-reactions': [{
            dependencies: ['.isFixed'],
            fulfill: {
              schema: {
                'x-disabled': '{{ !$deps[0] }}'
              }
            }
          }]
        }
      }
    }
  }
};
var ExpressionSetter = exports.ExpressionSetter = function ExpressionSetter(props) {
  var form = props.form,
    config = props.config,
    selected = props.selected;
  // const [selectedKeys, setSelectedKeys] = useState<string[]>([]);
  var onClick = function onClick(event, item) {
    console.log('selected', item, item.pos.split('-').length > 3);
    if (item.key && item.pos.split('-').length > 3) {
      form.setValues((0, _extends2["default"])({}, form.values, {
        value: (form.values.value || '') + item.key
      }));
    }
  };
  var systemVariableOptionsTree = config.get('systemVariableOptionsTree', []);
  var variables = selected.getVariables();
  console.log('----ExpressionSetter variables', variables);
  var variableOptionsTree = [{
    title: '项目 (Part)',
    key: 'part',
    children: [].concat(systemVariableOptionsTree, variables.length ? [{
      title: '状态变量',
      key: 'status',
      children: variables
    }] : [])
  }];
  return /*#__PURE__*/React.createElement(_row["default"], null, /*#__PURE__*/React.createElement(_col["default"], {
    span: 16
  }, /*#__PURE__*/React.createElement(_antdV.Form, {
    form: form,
    colon: false,
    labelCol: 8,
    wrapperCol: 16,
    feedbackLayout: "terse"
  }, /*#__PURE__*/React.createElement(SchemaField, {
    schema: schema
  }))), /*#__PURE__*/React.createElement(_col["default"], null, /*#__PURE__*/React.createElement(_tree["default"], {
    treeData: variableOptionsTree,
    selectable: false,
    defaultExpandAll: true,
    onClick: onClick
  })));
};